from git import Repo, Submodule;
from gitdb import exc;
import argparse;
import ConfigParser;
import os.path;
import pkg_resources;
import string;

SUBMODULE_PATH = ".git-standards";
INI_FILE_PATH = ".git-standards.ini";
DEFAULT_GIT_STANDARDS_OWNER = "rwhogg";
DEFAULT_GIT_STANDARDS_REPONAME = "git-standards";
DEFAULT_GIT_STANDARDS_REPO = DEFAULT_GIT_STANDARDS_OWNER + DEFAULT_GIT_STANDARDS_REPONAME;


def get_yes_no_answer(question):
    ans = "";
    while ans != "Y" and ans != "N":
        ans = string.upper(raw_input(question + " (Y/N) "));
    return ans;


def error(message):
    print "Error: " + message;
    exit(1);


class GitStandardize:

    def __init__(self, repoPath):
        self.repo = Repo(repoPath);
        try:
            self.submodule = self.repo.submodule(SUBMODULE_PATH);
        except ValueError:
            # that's fine, ignore
            self.submodule = None;
        except exc.BadName:
            error("You have not made your first commit yet!");

        self.configure();

    def configure(self):
        if not os.path.isfile(INI_FILE_PATH):
            return;
        with open(INI_FILE_PATH) as config_file:
            config_parser = ConfigParser.ConfigParser();
            config_parser.readfp(config_file);
            self.url = config_parser.get("git-standardize", "url");
            # FIXME: more

    def init(self):
        if self.submodule:
            error("This is already a GitStandardize-enabled repo!");

        have_standards = get_yes_no_answer("Do you already have a git-standards repo?");
        if have_standards == "Y":
            git_standards_location = raw_input("Where is it? ");
        else:
            like_to_fork = get_yes_no_answer("Would you like to fork " + DEFAULT_GIT_STANDARDS_REPO + "?");
            if like_to_fork == "Y":
                # FIXME
                pass;
            else:
                print "Please create a git-standards repo and run this task again."
                exit(1);
        print "Here are your settings:";
        config = "[git-standardize]\nurl: " + git_standards_location + "\n";
        # print "project_types: " + project_types;
        # FIXME create a proper object
        print config;
        okay = get_yes_no_answer("Are you okay with these settings?");
        if okay == "Y":
            with open(INI_FILE_PATH, "w") as config_file:
                config_file.write(config);
            print "Your settings have been written!";

    def add(self):
        if self.submodule:
            error("The git-standards submodule already exists!");
        self.submodule = Submodule.add(self.repo, SUBMODULE_PATH, SUBMODULE_PATH, url = self.url);
        self.update();

    def update(self):
        if not self.submodule:
            error("The git-standards submodule does not exist!");
        self.submodule.update();
        self.symlink();

    def should_symlink(self, path):
        # FIXME
        return path != ".git" and path != "git-standards.ini" and path != "README.md";

    def symlink(self):
        # FIXME: will want project type support eventually
        if not self.submodule:
            error("The git-standards submodule does not exist!");
        paths = os.listdir(SUBMODULE_PATH);
        for path in paths:
            if self.should_symlink(path):
                try:
                    os.symlink(os.path.join(SUBMODULE_PATH, path), path);
                except OSError:
                    # just ignore overwrite errors
                    pass;

    def get_submodule_as_repo(self):
        return Repo(SUBMODULE_PATH);

    def commit(self):
        standards_repo = self.get_submodule_as_repo();
        standards_repo.git.add(".");
        standards_repo.git.commit("-a", "-m", '"Bump git-standards"');
        self.repo.git.commit(SUBMODULE_PATH, "-m", "Update git-standards");

    def push(self):
        standards_repo = self.get_submodule_as_repo();
        standards_repo.git.push();


def init():
    GitStandardize(".").init();


def add():
    GitStandardize(".").add();


def update():
    GitStandardize(".").update();


def commit():
    GitStandardize(".").commit();


def push():
    GitStandardize(".").push();

def version():
    print "git-standardize " + pkg_resources.require("GitStandardize")[0].version;
    print "Copyright (C) 2016, Bob W. Hogg <rwhogg@linux.com>. All rights reserved."
    print "See the file LICENSE.txt from the source distribution for copying conditions.";
    exit(0);

# set up the command-line parser
def init_parser():
    parser = argparse.ArgumentParser();
    subparsers = parser.add_subparsers(help = "Possible actions");
    init_parser = subparsers.add_parser("init", help = "Initialize this repo for GitStandardize");
    init_parser.set_defaults(func = init);
    add_parser = subparsers.add_parser("add", help = "Clone the git-standards repo");
    add_parser.set_defaults(func = add);
    update_parser = subparsers.add_parser("update", help = "Update the git-standards repo");
    update_parser.set_defaults(func = update);
    commit_parser = subparsers.add_parser("commit", help = "Commit changes to the git-standards repo");
    commit_parser.set_defaults(func = commit);
    push_parser = subparsers.add_parser("push", help = "Push changes to the upstream git-standards repo");
    push_parser.set_defaults(func = push);
    version_parser = subparsers.add_parser("version", help = "Display version information");
    version_parser.set_defaults(func = version);
    return parser;

def main():
    parser = init_parser();
    args = parser.parse_args();
    if parser.version:
        version();
    else:
        args.func();
